#
#				   lib.make
#
# Inputs from Makefile.preamble or Makefile.postamble (no defaults):
#	OTHER_CFLAGS - Flags passed to compiler (in addition to -g, -O, etc.)
#	OTHER_LDFLAGS - Flags passed to ld (in addition to -ObjC, etc.)
#	OTHER_OFILES - Additional relocatables that may be linked in
#	OTHER_PRODUCT_DEPENDS - Other dependencies of this project
#	OTHER_EARLY_TARGETS - Built before building subprojects/bundles
#	OTHER_SOURCEFILES - Other source files maintained by .pre/postamble
#	OTHER_GARBAGE - Additional files to be removed by make 'clean'
#
#	DSTROOT - Virtual root directory prepended to $(INSTALLDIR)
#
# Inputs from Makefile.preamble:
#	PRECOMPS - Precompiled headers that should be built before compilation
#	OTHER_INITIAL_TARGETS - Targets to be built before subprojects, etc.
#
# Inputs from Makefile.postamble (with defaults set in common.make):
#	INSTALL_AS_USER - User to chown lib to at installation (default root)
#	INSTALL_AS_GROUP - Grp to chgrp lib to at installation (default wheel)
#	INSTALL_PERMISSIONS - If set, 'install' chmod's executable to this.
#	MAKEFILES - Makefiles this project depends on (default = Makefile)
#
#	RELOCATABLE_STRIP_OPTS - Opts to strip for bundles (default = -x -u)
#	DYLD_BIN_STRIP_OPTS - Opts to strip for libs w/ bundles (def. = -A -n)
#	BIN_STRIP_OPTS - Opts to strip for libs w/o bundles (no default, but
#	   developers doing their own dynamic loading should set this to
#	   $(DYLD_BIN_STRIP_OPTS)
#
# Inputs from Makefile (avoid setting these, ProjectBuilder controls them):
#	NAME - name of project
#
#	CLASSES - Class implementation files in project.
#	HFILES - Header files in project.
#	MFILES - Other Objective-C source files in project.
#	CFILES - Other C source files in project.
#	PSWFILES - .psw files in the project
#	PSWMFILES - .pswm files in the project
#	SUBPROJECTS - Subprojects of this project
#	OTHERSRCS - Other miscellaneous sources of this project
#	OTHERLINKED - Source files not matching a standard source extention
#
#	OTHERLINKEDOFILES - Other relocatable files to (always) link in.
#
#	TMMAKEFILEDIR - Directory in which to find this generic set of Makefiles
#	INSTALLDIR - Directory lib will be installed into by 'install' target
#	INSTALLFLAGS - not used
#	SOURCEMODE - not used
#
# Outputs from this file to common.make:
#	OFILE_DIR
#
#

include $(TMMAKEFILEDIR)/common.make

all:: lib


lib::
	@($(MKDIRS) . ; \
	$(MAKE) project \
		"PRODUCT_ROOT = ." \
		"PRODUCT = lib$(NAME).a" \
		"SRCDIR=`pwd`" \
		"OFILE_DIR=obj" \
		"CFLAGS = -O -Wall $(CFLAGS) $(OTHER_CFLAGS)" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)

debug::
	@($(MKDIRS) . ; \
	$(MAKE) project \
		"PRODUCT_ROOT = ." \
		"PRODUCT = lib$(NAME)_g.a" \
		"SRCDIR=`pwd`" \
		"OFILE_DIR=debug_obj" \
		"CFLAGS = -g -Wall -DDEBUG $(CFLAGS) $(OTHER_CFLAGS)" \
		"LIBS = $(DEBUG_LIBS)" "OTHER_LIBS = $(OTHER_DEBUG_LIBS)" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)

profile::
	@($(MKDIRS) . ; \
	$(MAKE) project \
		"PRODUCT_ROOT = ." \
		"PRODUCT = lib$(NAME)_p.a" \
		"SRCDIR=`pwd`" \
		"OFILE_DIR=profile_obj" \
		"CFLAGS = -g -pg -O -Wall -DPROFILE $(CFLAGS) $(OTHER_CFLAGS)" \
		"LIBS = $(PROF_LIBS)" "OTHER_LIBS = $(OTHER_PROF_LIBS)" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)


# Note: The order of the following project dependencies is very important.  subcomponents must be after $(PROJECT_OFILES) in case a subcomponent relies upon something (e.g. generated .h file) at the top level, but all subprojects must be built (hopefully by one invocation of make in that subdirectory) before the $(PRODUCT) can be linked.

project::   $(INITIAL_TARGETS) subcomponents $(PRODUCT)

$(PRODUCT): $(PRODUCT_DEPENDS) $(OTHER_PRODUCT_DEPENDS)
	@(if [ -f "$(VERSIONFILE)" ] ; then \
	    if [ -f CVS/Entries ] ; then \
	      if $(CVS) -n update "$(VERSIONFILE)" | $(GREP) -s '^[M]' ; then \
		echo -n > /dev/null ; \
	      elif $(CVS) -n update -l | $(GREP) -v "$(VERSIONFILE)" \
		      | $(GREP) -s '^[MA]' ; then \
		echo "Updating version file." ; \
		echo >> "$(SRCDIR)/$(VERSIONFILE)" ; \
	      fi ; \
	    fi ; \
	    versionf=$(OFILE_DIR)/`echo "$(VERSIONFILE)" \
				   | sed 's?\.[mc]??'`.o ; \
	    cmd="$(CC) $(CFLAGS) $(OBJCFLAGS) -c "$(SRCDIR)/$(VERSIONFILE)" \
		-o $$versionf" ; \
	    echo $$cmd ; $$cmd ; \
	  else \
	    versionf="" ; \
	  fi ; \
	  $(AR) ru $@ $(OFILES) $$versionf $(OTHER_OFILES); \
	  echo "$(RANLIB) -s $@"; \
	  $(RANLIB) -s $@ \
	)

$(DSTROOT)$(INSTALLDIR):
	$(MKDIRS) $(DSTROOT)$(INSTALLDIR)
	$(MKDIRS) $(DSTROOT)$(INSTALLDIR)/lib
	$(MKDIRS) $(DSTROOT)$(INSTALLDIR)/include

before_install:: $(DSTROOT)$(INSTALLDIR)
	$(RM) -f $(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME).a
	$(RM) -f $(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME)_g.a
	$(RM) -f $(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME)_p.a
	$(RM) -rf $(DSTROOT)$(INSTALLDIR)/include/$(NAME)
	$(MKDIRS) $(DSTROOT)$(INSTALLDIR)/include/$(NAME)

after_install::


subinstall:: lib debug profile before_install $(OTHER_INSTALL_DEPENDS) subcomponents
	$(CP) -p lib$(NAME).a $(DSTROOT)$(INSTALLDIR)/lib
	$(CP) -p lib$(NAME)_g.a $(DSTROOT)$(INSTALLDIR)/lib
	$(CP) -p lib$(NAME)_p.a $(DSTROOT)$(INSTALLDIR)/lib
	$(CP) -p $(INSTALLHDRS) $(DSTROOT)$(INSTALLDIR)/include/$(NAME)
	-$(CHOWN) -R -f $(INSTALL_AS_USER).$(INSTALL_AS_GROUP)	\
		$(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME).a	\
		$(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME)_g.a	\
		$(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME)_p.a	\
		$(DSTROOT)$(INSTALLDIR)/include/$(NAME)
	-$(CHMOD) -R -f go-w					\
		$(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME).a	\
		$(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME)_g.a	\
		$(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME)_p.a	\
		$(DSTROOT)$(INSTALLDIR)/include/$(NAME)
	if [ "$(INSTALL_PERMISSIONS)" != "" ] ; then \
		$(CHMOD) -R -f $(INSTALL_PERMISSIONS) \
		$(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME).a	\
		$(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME)_g.a	\
		$(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME)_p.a	\
		$(DSTROOT)$(INSTALLDIR)/include/$(NAME);	\
	fi
	$(RANLIB) -s -t $(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME).a
	$(RANLIB) -s -t $(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME)_g.a
	$(RANLIB) -s -t $(DSTROOT)$(INSTALLDIR)/lib/lib$(NAME)_p.a
	@$(MAKE) after_install


help::
	@$(ECHO) 'The following targets are available:'
	@$(ECHO)
	@$(ECHO) ' all	      Makes the lib target (below).  This is the default target.'
	@$(ECHO)
	@$(ECHO) ' lib	      Compiles and links an optimized (and debuggable) version of the'
	@$(ECHO) '	      project, ready to install, into <ProjectName>.'
	@$(ECHO)
	@$(ECHO) ' debug      Compiles (with all warnings and -DDEBUG on) and links a '
	@$(ECHO) '	      fully-debuggable version of the project into a <ProjectName>.debug'
	@$(ECHO) '	      "lib wrapper".'
	@$(ECHO)
	@$(ECHO) ' profile    Compiles (with all warnings and -DPROFILE on) and links a'
	@$(ECHO) '	      version of the project into <ProjectName>.profile.'
	@$(ECHO) '	      "lib wrapper" that will, when run, generate a gprof report.'
	@$(ECHO)
	@$(ECHO) ' depend     Generates an optional Makefile.dependencies file for every level'
	@$(ECHO) '	      of the project to reflect the complete dependency graph'
	@$(ECHO) '	      (including headers).'
	@$(ECHO)
	@$(ECHO) ' install    Builds (if needed) and copies <ProjectName>.lib into the'
	@$(ECHO) '	      installation directory, setting permissions and owners as'
	@$(ECHO) '	      appropriate.'
	@$(ECHO)
	@$(ECHO) ' installsrc Installs the source files for the project into the directory'
	@$(ECHO) '	      specified in the SRCROOT variable in a command-line argument.'
	@$(ECHO) '	      Note: the target directory will be removed and recreated.'
	@$(ECHO)
	@$(ECHO) ' clean      Removes all files in the project derived from previous builds.'
	@$(ECHO)
	@$(ECHO) ' help	      Lists these targets with their descriptions.'
	@$(ECHO)
	@$(ECHO) 'For more details, see the comments in ./lib.make.'
